<?php

namespace Drupal\ptools\Entity;

use EntityDrupalWrapper;
use EntityListWrapper;
use EntityMetadataWrapperException;

/**
 * Entity metadata wrapper for a Community group node object.
 */
class EntityWrapper extends EntityDrupalWrapper {

  /**
   * Instantiates a new object based upon the entity
   *
   * @param $entity
   *   The entity object or its ID.
   *
   * @return
   *   A new object based upon the called entity
   */
  public static function wrap($entity) {
    return new static($entity);
  }

  /**
   * Checks whether the entity is new.
   *
   * @retrun bool
   *   TRUE if the entity is new, FALSE otherwise.
   */
  public function isNew() {
    $entity = $this->value();
    if (!empty($entity->is_new)) {
      return TRUE;
    }
    $id = $this->getIdentifier();
    return !isset($id);
  }

  /**
   * Renders a view mode for the given entity.
   *
   * @param $view_mode
   *   A view mode as used by this entity type, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   * @param $page
   *   (optional) If set will control if the entity is rendered: if TRUE
   *   the entity will be rendered without its title, so that it can be embedded
   *   in another context. If FALSE the entity will be displayed with its title
   *   in a mode suitable for lists.
   *   If unset, the page mode will be enabled if the current path is the URI
   *   of the entity, as returned by entity_uri().
   *   This parameter is only supported for entities which controller is a
   *   EntityAPIControllerInterface.
   *
   * @return string
   *   The rendered content.
   */
  public function render($view_mode = 'full', $langcode = NULL, $page = NULL) {
    $build = $this->view($view_mode, $langcode, $page);
    return render($build);
  }

  /**
   * Safely gets an item list.
   *
   * @param string $field_name
   *   The field name.
   *
   * @return \EntityListWrapper
   *   A list of field items.
   */
  protected function safeGetItems($field_name) {
    try {
      return $this->get($field_name);
    }
    catch (EntityMetadataWrapperException $e) {
      return new EntityListWrapper('', []);
    }
  }

  /**
   * Safely gets a field value.
   *
   * @param string $field_name
   *   The field name.
   *
   * @return mixed
   *   The field value or NULL if not available.
   */
  protected function safeGetValue($field_name) {
    /** @var \EntityMetadataWrapper $item */
    try {
      $item = $this->get($field_name);
      return $item->value();
    }
    catch (EntityMetadataWrapperException $e) {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function __get($name) {
    $entity = $this->value();
    if (isset($entity->{$name}) && is_scalar($entity->{$name})) {
      $get = $entity->{$name};
      return $get;
    }
    return parent::__get($name);
  }

  /**
   * Does the field have a value?
   *
   * @param string $field_name
   *   The name of the string
   *
   * @return bool
   */
  public function hasValue($field_name) {
    $value = $this->safeGetValue($field_name);
    return isset($value);
  }
}
