<?php

namespace Drupal\ptools\Queue;

use Drupal\ptools\Plugin\PluginBase;

/**
 * Base class for queue handler plugins.
 */
abstract class QueueHandlerBase extends PluginBase implements QueueHandlerInterface {

  /**
   * The queue name.
   *
   * @var string
   */
  protected $queueName;

  /**
   * The handled queue.
   *
   * @var \DrupalQueueInterface
   */
  protected $queue;

  /**
   * {@inheritdoc}
   */
  public function setQueueContext($queue_name, \DrupalQueueInterface $queue) {
    $this->queueName = $queue_name;
    $this->queue = $queue;
  }

  /**
   * {@inheritdoc}
   */
  public function queueItems(array $items = NULL) {
    if (isset($items)) {
      $processed = $this->createItems($items);
    }
    else {
      $processed = 0;
      $total = $this->getPendingItemsCount();
      while ($processed < $total) {
        $items = $this->fetchPendingItems($processed);
        $processed += $this->createItems($items);
        $this->log(format_string('-> @count / @total processed', ['@count' => $processed, '@total' => $total]));
      }
    }
    $this->log(format_string('@count items added to the queue.', ['@count' => $processed]));
  }

  /**
   * Returns the total number of pending items.
   *
   * @return int
   *   The pending items count.
   */
  abstract protected function getPendingItemsCount();

  /**
   * Fetches a batch of pending items to be queued.
   *
   * @param int $offset
   *   (optional) The offset index to start from. Defaults to 0.
   *
   * @return mixed[]
   *   An array of pending items.
   */
  abstract protected function fetchPendingItems($offset = 0);

  /**
   * Creates the specified queue items.
   *
   * @param mixed[] $items
   *   An array of queue items.
   *
   * @return int
   *   The number of processed items.
   */
  protected function createItems(array $items) {
    $this->queue->createQueue();
    foreach ($items as $item) {
      $this->queue->createItem([
        'item' => $item,
        'attempts' => $this->definition['max_attempts'],
      ]);
    }
    return count($items);
  }

  /**
   * {@inheritdoc}
   */
  public function processItems($limit = -1, $time_limit = NULL) {
    $processed = 0;
    $failed = 0;
    $count = intval($limit) ?: -1;
    if (isset($time_limit)) {
      $end = time() + $time_limit;
    }

    while ($count-- !== 0 && (!isset($end) || time() < $end) && ($item = $this->queue->claimItem($this->definition['lease_time']))) {
      try {
        $this->processItem($item->data['item']);
        $this->queue->deleteItem($item);
        $processed++;
      }
      catch (\Exception $e) {
        // In case of exception log it and leave the item in the queue to be
        // processed again later.
        watchdog_exception('ptools_queue', $e);

        // Before giving up, we queue up the item again until we reach the
        // maximum number of attempts.
        if ($item->data['attempts'] > 1) {
          $this->queue->deleteItem($item);
          $item->data['attempts']--;
          $this->queue->createItem($item->data);
        }
        else {
          $args = [
            '@name' => $this->queueName,
            '@item_id' => $item->item_id,
            '@count' => $this->definition['max_attempts'],
            '@item' => print_r($item->data['item'], TRUE),
          ];
          watchdog('ptools_queue', 'Item "@item_id" of queue "@name" could not be processed after @count attempts: <pre>@item</pre>', $args, WATCHDOG_ERROR);

          // If after all the specified attempts the item is still failing, we
          // need to remove the item permanently, if the queue is expected to be
          // empty at the end of the process.
          if ($this->definition['clear_failed']) {
            $this->queue->deleteItem($item);
          }

          $failed++;
        }
      }
    }

    $args = [
      '@processed_count' => $processed,
      '@failed_count' => $failed,
      '@total' => $this->queue->numberOfItems(),
    ];
    $this->log(format_string('@processed_count queue items processed, @failed_count failed, @total remaining.', $args));

    return $processed;
  }

  /**
   * Processes a single item.
   *
   * @param mixed $data
   *   The queue item data.
   */
  abstract protected function processItem($data);

  /**
   * {@inheritdoc}
   */
  public function clearQueue() {
    $processed = $this->queue->numberOfItems();
    $this->queue->deleteQueue();
    $this->log(format_string('@count items removed from queue.', ['@count' => $processed]));
    return $processed;
  }

  /**
   * Logs a message.
   *
   * @param string $message
   *   A message string.
   */
  protected function log($message) {
    // @todo Inject logger instead.
    if (function_exists('drush_log')) {
      drush_log($message, 'ok');
    }
  }

}
