<?php

namespace Drupal\ptools\Utils;

use Drupal\ptools\SingletonTrait;

/**
 * Class collecting date-related helper methods.
 */
class DateUtils {
  use SingletonTrait;

  /**
   * The date configuration.
   *
   * @var array
   */
  protected $config;

  /**
   * The current user.
   *
   * @var object
   */
  protected $currentUser;

  /**
   * DateUtils constructor.
   *
   * @param array $config
   *   The date configuration.
   * @param object $current_user
   *   The current user.
   */
  public function __construct($config, $current_user) {
    $this->config = $config;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create() {
    $config = [
      'date_default_timezone' => 'UTC',
      'configurable_timezones' => 1,
      'date_first_day' => 0,
    ];
    foreach ($config as $key => $default_value) {
      $config[$key] = variable_get($key, $default_value);
    }
    return new static(
      $config,
      $GLOBALS['user']
    );
  }

  /**
   * Returns a date/time object for the specified PHP date time specifier.
   *
   * @param string $time
   *   (optional) A PHP date time specifier. Defaults to now.
   * @param string|null $timezone
   *   (optional) A timezone identifier. Defaults to none.
   *
   * @return \DateTime
   *   A date/time object.
   */
  public function dateFromTime($time = 'now', $timezone = NULL) {
    $date_timezone = new \DateTimeZone(isset($timezone) ? $timezone : $this->getCurrentTimezone());
    $date = new \DateTime($time, $date_timezone);
    return $date;
  }

  /**
   * Returns a date/time object for the specified timestamp.
   *
   * @param int $timestamp
   *   A Unix timestamp.
   * @param string|null $timezone
   *   (optional) A timezone identifier. Defaults to none.
   *
   * @return \DateTime
   *   A date/time object.
   */
  public function dateFromTimestamp($timestamp, $timezone = NULL) {
    $date = \DateTime::createFromFormat('U', $timestamp);
    $date_timezone = new \DateTimeZone(isset($timezone) ? $timezone : $this->getCurrentTimezone());
    $date->setTimezone($date_timezone);
    return $date;
  }

  /**
   * Returns the week start timestamp for the specified time.
   *
   * @param string $time
   *   A PHP time.
   * @param string|null $timezone
   *   (optional) A timezone identifier. Defaults to none.
   *
   * @return int
   *   A Unix timestamp.
   */
  public function getWeekStartTimestamp($time, $timezone = NULL) {
    $date = new \DateTime($time);
    $date_timezone = new \DateTimeZone(isset($timezone) ? $timezone : $this->getCurrentTimezone());
    $date->setTimezone($date_timezone);
    $week_start_date = $this->getWeekStart($date);
    return intval($week_start_date->format('U'));
  }

  /**
   * Returns the week start for the specified time.
   *
   * @param \DateTime $date
   *   (optional) A date/time object. Defaults to one representing the current
   *   time.
   *
   * @return \DateTime
   *   A date/time object representing the week start.
   */
  public function getWeekStart(\DateTime $date = NULL) {
    if (!isset($date)) {
      $date_timezone = new \DateTimeZone($this->getCurrentTimezone());
      $date = new \DateTime('now', $date_timezone);
    }
    $day = $this->config['date_first_day'];
    $week_number = $date->format('W');
    // ISO-8601 week number week in the year considers weeks starting on Monday,
    // if the first day is Sunday the "next" week should be selected.
    if ($day < 1) {
      $week_number++;
    }
    $date->setISODate($date->format('Y'), $week_number, $day);
    $date->setTime(0, 0, 0);
    return $date;
  }

  /**
   * Returns the current timezone.
   *
   * @return string
   *   A timezone identifier.
   */
  public function getCurrentTimezone() {
    return $this->getUserTimezone($this->currentUser);
  }

  /**
   * Returns the current timezone.
   *
   * @param object $account
   *   A user account object.
   *
   * @return string
   *   A timezone identifier.
   */
  public function getUserTimezone($account) {
    if ($account && $this->config['configurable_timezones'] && !empty($account->timezone)) {
      $timezone = $account->timezone;
    }
    else {
      $timezone = $this->getDefaultTimezone();
    }
    return $timezone;
  }

  /**
   * Returns the default timezone.
   *
   * @return string
   *   A timezone identifier.
   */
  public function getDefaultTimezone() {
    return $this->config['date_default_timezone'];
  }

}
